package com.kogenta.kettle

import androidx.annotation.NonNull
import com.kogenta.kettle.common.logging.LogLevel
import com.kogenta.kettle.common.model.DemographicProfile
import com.kogenta.kettle.sdk.Kettle
import com.kogenta.kettle.sdk.KettleConsent
import com.kogenta.kettle.sdk.KettleModule

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** KettlePlugin */
class KettlePlugin: FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "com.kogenta.kettle/kettle")
    channel.setMethodCallHandler(this)
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    try {
      when(call.method) {
        "metric" -> {
          result.success(true)
        }
        "start" -> {
          startKettle(call, result)
        }
        "stop" -> {
          stopKettle(call, result)
        }
        "grant" -> {
          grantConsents(call, result)
        }
        "revoke" -> {
          revokeConsents(call, result)
        }
        "sendDemographicProfile" -> {
          sendDemographicProfile(call, result)
        }
        "addCustomEvent" -> {
          addCustomEvent(call, result)
        }
        "setMetadata" -> {
          setMetadata(call, result)
        }
        "setLogLevel" -> {
          setLogLevel(call, result)
        }
        "getIdentifier" -> {
          getIdentifier(call, result)
        }
        "setPushToken" -> {
          setPushToken(call, result)
        }
        "setExternalId" -> {
          setExternalId(call, result)
        }
        "isStarted" -> {
          isStarted(call, result)
        }
        "getPrivacyDashboardUrl" -> {
          getPrivacyDashboardUrl(call, result)
        }
        "getSustainabilityDashboard" -> {
          getSustainabilityDashboard(call, result)
        }
        "getApiKey" -> {
          getApiKey(call, result)
        }
        "deleteCollectedData" -> {
          deleteCollectedData(call, result)
        }
        "getGrantedConsents" -> {
          getGrantedConsents(call, result)
        }
        "getEnabledModules" -> {
          getEnabledModules(call, result)
        }
        "getActivityEvents" -> {
          getActivityEvents(call, result)
        }
        "anonymizeLocation" -> {
          anonymizeLocation(call, result)
        }
      }
    } catch (e: Exception) {
      result.error("KTLERR-0", "KTLException", e.localizedMessage)
    }
  }

  private fun anonymizeLocation(call: MethodCall, result: Result) {
    val latitude: Double? = call.argument("latitude")
    val longitude: Double? = call.argument("longitude")
    if (latitude == null || longitude == null || (latitude == 0.0 && longitude == 0.0)) {
      return result.error("KTLERR-9", "Longitude and Latitude can not be null, empty or default.", null)
    }
    Kettle.anonymizeLocation(latitude, longitude) Anonymize@ {
      return@Anonymize result.success(it)
    }
  }

  private fun getActivityEvents(call: MethodCall, result: Result) {
    val fromTime: Long? = call.argument("fromTime")
    val toTime: Long? = call.argument("toTime")
    if (fromTime == null || toTime == null) {
      return result.error("KTLERR-10", "fromTime and toTime must be supplied in milliseconds since epoch format", null)
    }
    Kettle.getActivityEvents(fromTime, toTime) GetActivities@ {
      return@GetActivities result.success(it)
    }
  }

  private fun getEnabledModules(call: MethodCall, result: Result) {
    val modules = arrayListOf<Int>()
    for (module in Kettle.enabledModules) {
      when (module) {
        KettleModule.LOCATION -> modules.add(1)
        KettleModule.BLUETOOTH -> modules.add(2)
        KettleModule.ACTIVITY -> modules.add(3)
      }
    }
    return result.success(modules.toList())
  }

  private fun getGrantedConsents(call: MethodCall, result: Result) {
    val consents = arrayListOf<Int>()
    for (consent in Kettle.grantedConsents) {
      when (consent) {
        KettleConsent.SURVEYS -> consents.add(1001)
        KettleConsent.ANALYTICS -> consents.add(1002)
        KettleConsent.CAMPAIGNS -> consents.add(1003)
        KettleConsent.ADS -> consents.add(1004)
      }
    }
    return result.success(consents.toList())
  }

  private fun deleteCollectedData(call: MethodCall, result: Result) {
    Kettle.deleteCollectedData ClearPII@ { return@ClearPII result.success(it) }
  }

  private fun getApiKey(call: MethodCall, result: Result) {
    return result.success(Kettle.apiKey)
  }

  private fun getSustainabilityDashboard(call: MethodCall, result: Result) {
    return result.success(Kettle.sustainabilityDashboardUrl)
  }

  private fun getPrivacyDashboardUrl(call: MethodCall, result: Result) {
    return result.success(Kettle.privacyDashboardUrl)
  }

  private fun isStarted(call: MethodCall, result: Result) {
    return result.success(Kettle.started)
  }

  private fun setExternalId(call: MethodCall, result: Result) {
    val externalId: String? = call.argument("externalId")
    if (externalId.isNullOrEmpty()) {
      return result.error("KTLERR-8", "External identifier can not be null or empty.", null)
    }
    Kettle.externalId = externalId
    return result.success(true)
  }

  private fun setPushToken(call: MethodCall, result: Result) {
    val pushToken: String? = call.argument("pushToken")
    if (pushToken.isNullOrEmpty()) {
      return result.error("KTLERR-7", "Push token can not be null or empty.", null)
    }
    Kettle.pushToken = pushToken
    return result.success(true)
  }

  private fun getIdentifier(call: MethodCall, result: Result) {
    return result.success(Kettle.identifier)
  }

  private fun setLogLevel(call: MethodCall, result: Result) {
    when (call.argument("logLevel") as Int?) {
      0 -> Kettle.logLevel = LogLevel.EMPTY
      1 -> Kettle.logLevel = LogLevel.TRACE
      2 -> Kettle.logLevel = LogLevel.VERBOSE
      3 -> Kettle.logLevel = LogLevel.WARNING
      4 -> Kettle.logLevel = LogLevel.INFO
      else -> Kettle.logLevel = LogLevel.INFO
    }
    return result.success(true)
  }

  private fun setMetadata(call: MethodCall, result: Result) {
    val type: String? = call.argument("type")
    val json: String? = call.argument("json")
    if (type.isNullOrEmpty() || json.isNullOrEmpty()) {
      return result.error("KTLERR-6", "Type and json can not be null or empty. Supply both.", null)
    }
    Kettle.setMetadata(type, json)
    return result.success(true)
  }

  private fun addCustomEvent(call: MethodCall, result: Result) {
    val type: String? = call.argument("type")
    val json: String? = call.argument("json")
    if (type.isNullOrEmpty() || json.isNullOrEmpty()) {
      return result.error("KTLERR-5", "Type and json can not be null or empty. Supply both.", null)
    }
    Kettle.addCustomEvent(type, json)
    return result.success(true)
  }

  private fun sendDemographicProfile(call: MethodCall, result: Result) {
    val birthYear: Int? = call.argument("birthYear")
    val gender: String? = call.argument("gender")
    if (birthYear == null || gender.isNullOrEmpty()) {
      return result.error("KTLERR-4", "Gender and birth year can not be null or empty. Supply at least one.", null)
    }

    val finalGender = when (gender) {
      "male" -> DemographicProfile.Gender.MALE
      "female" -> DemographicProfile.Gender.FEMALE
      "unknown" -> DemographicProfile.Gender.UNKNOWN
      else -> DemographicProfile.Gender.UNKNOWN
    }

    Kettle.sendDemographicProfile(DemographicProfile(birthYear = birthYear, gender = finalGender))
    return result.success(true)
  }

  private fun revokeConsents(call: MethodCall, result: Result) {
    val consents = call.argument<List<Int>>("consents")
    if (consents.isNullOrEmpty()) {
      return result.error("KTLERR-3", "At least one consent must be revoked.", null)
    }

    val kettleConsents = arrayListOf<KettleConsent>()
    for (consent in consents) {
      when (consent) {
        1001 -> kettleConsents.add(KettleConsent.SURVEYS)
        1002 -> kettleConsents.add(KettleConsent.ANALYTICS)
        1003 -> kettleConsents.add(KettleConsent.CAMPAIGNS)
        1004 -> kettleConsents.add(KettleConsent.ADS)
      }
    }

    Kettle.revoke(kettleConsents.toTypedArray())
    return result.success(true)
  }

  private fun grantConsents(call: MethodCall, result: Result) {
    val consents = call.argument<List<Int>>("consents")
    if (consents.isNullOrEmpty()) {
      return result.error("KTLERR-2", "At least one consent must be granted.", null)
    }

    val kettleConsents = arrayListOf<KettleConsent>()
    for (consent in consents) {
      when (consent) {
        1001 -> kettleConsents.add(KettleConsent.SURVEYS)
        1002 -> kettleConsents.add(KettleConsent.ANALYTICS)
        1003 -> kettleConsents.add(KettleConsent.CAMPAIGNS)
        1004 -> kettleConsents.add(KettleConsent.ADS)
      }
    }

    Kettle.grant(kettleConsents.toTypedArray())
    return result.success(true)
  }

  private fun stopKettle(call: MethodCall, result: Result) {
    val modules = call.argument<List<Int>>("modules")
    if (modules.isNullOrEmpty()) {
      Kettle.stop()
      return result.success(true)
    }

    val kettleModules = arrayListOf<KettleModule>()
    for (module in modules) {
      when (module) {
        1 -> kettleModules.add(KettleModule.LOCATION)
        2 -> kettleModules.add(KettleModule.BLUETOOTH)
        3 -> kettleModules.add(KettleModule.ACTIVITY)
      }
    }

    Kettle.stop(kettleModules.toTypedArray())
    return result.success(true)
  }

  private fun startKettle(call: MethodCall, result: Result) {
    val modules = call.argument<List<Int>>("modules")
    if (modules.isNullOrEmpty()) {
      return result.error("KTLERR-1", "Kettle cannot be started without modules.", null)
    }

    val kettleModules = arrayListOf<KettleModule>()
    for (module in modules) {
      when (module) {
        1 -> kettleModules.add(KettleModule.LOCATION)
        2 -> kettleModules.add(KettleModule.BLUETOOTH)
        3 -> kettleModules.add(KettleModule.ACTIVITY)
      }
    }

    Kettle.start(kettleModules.toTypedArray())
    return result.success(true)
  }
}
