import Flutter
import UIKit
import KettleKit

public class KettlePlugin: NSObject, FlutterPlugin {
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "com.kogenta.kettle/kettle", binaryMessenger: registrar.messenger())
        let instance = KettlePlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        let method: String = call.method
        do {
            switch method {
            case "start": start(call, result)
            case "stop": stop(call, result)
            case "grant": grant(call, result)
            case "revoke": revoke(call, result)
            case "sendDemographicProfile": sendDemographicProfile(call, result)
            case "addCustomEvent": addCustomEvent(call, result)
            case "setMetadata": setMetadata(call, result)
            case "setLogLevel": setLogLevel(call, result)
            case "getIdentifier": getIdentifier(call, result)
            case "setPushToken": setPushToken(call, result)
            case "setExternalId": setExternalId(call, result)
            case "isStarted": isStarted(call, result)
            case "getPrivacyDashboardUrl": getPrivacyDashboardUrl(call, result)
            case "getSustainabilityDashboardUrl": getSustainabilityDashboardUrl(call, result)
            case "getApiKey": getApiKey(call, result)
            case "deleteCollectedData": deleteCollectedData(call, result)
            case "getGrantedConsents": getGrantedConsents(call, result)
            case "getEnabledModules": getEnabledModules(call, result)
            case "getActivityEvents": getActivityEvents(call, result)
            case "anonymizeLocation": anonymizeLocation(call, result)
            case "metric": result(true)
            default: result(true)
            }
        } catch {
            result(FlutterError(code: "KTLERR-0", message: "KTLException", details: error.localizedDescription))
        }
    }
    
    private func anonymizeLocation(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        result("UNKNOWN")
    }
    
    private func getActivityEvents(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        Kettle.shared.fetchTrips { trips in
            result(trips)
        }
    }
    
    private func getEnabledModules(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        var enabledModules: [Int] = []
        if Kettle.shared.isEnabled(module: KettleModule.location) {
            enabledModules.append(1)
        }
        if Kettle.shared.isEnabled(module: KettleModule.bluetooth) {
            enabledModules.append(2)
        }
        if Kettle.shared.isEnabled(module: KettleModule.activity) {
            enabledModules.append(3)
        }
        result(enabledModules)
    }
    
    private func getGrantedConsents(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        result(
            Kettle.shared.grantedConsents.map { (consent) -> Int in
                switch consent {
                case .surveys: return 1001
                case .analytics: return 1002
                case .campaigns: return 1003
                case .ads: return 1004
                @unknown default:
                    return 0
                }
            }
        )
    }
    
    private func deleteCollectedData(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        Kettle.shared.deleteCollectedData { didRequestDeletion in
            result(didRequestDeletion)
        }
    }
    
    private func getApiKey(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        result("<UNKNOWN>")
    }
    
    private func getSustainabilityDashboardUrl(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        result("https://kogenta.com/")
    }
    
    private func getPrivacyDashboardUrl(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        result(Kettle.shared.privacyDashboardUrl)
    }
    
    private func isStarted(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        result(Kettle.shared.started)
    }
    
    private func setExternalId(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let args = call.arguments as? [String: Any] else {
            result(FlutterError(code: "KTLERR-4", message: "externalId can not be null or empty.", details: nil))
            return
        }
        
        if let externalId = args["externalId"] as? String {
            Kettle.shared.externalId = externalId
            result(true)
        } else {
            result(true)
        }
    }
    
    private func setPushToken(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let args = call.arguments as? [String: Any] else {
            result(FlutterError(code: "KTLERR-4", message: "pushToken can not be null or empty.", details: nil))
            return
        }
        
        if let pushToken = args["pushToken"] as? String {
            Kettle.shared.pushToken = pushToken
            result(true)
        } else {
            result(true)
        }
    }
    
    private func getIdentifier(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        // stub
        result(Kettle.shared.identifier)
    }
    
    private func setLogLevel(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        // stub - does not exist for iOS, done through config
        result(true)
    }
    
    private func setMetadata(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        // stub
        result(true)
    }
    
    private func addCustomEvent(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        // stub
        result(true)
    }
    
    private func sendDemographicProfile(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        // stub
        result(true)
    }
    
    private func revoke(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let args = call.arguments as? [String: Any] else {
            result(FlutterError(code: "KTLERR-3", message: "At least one consent must be provided.", details: nil))
            return
        }
        
        if let consents = args["consents"] as? [Int] {
            if consents.isEmpty {
                result(FlutterError(code: "KTLERR-3", message: "At least one consent must be provided.", details: nil))
                return
            }
            var kettleConsents: [KettleConsent] = []
            for consent in consents {
                switch consent {
                case 1001: kettleConsents.append(KettleConsent.surveys)
                case 1002: kettleConsents.append(KettleConsent.analytics)
                case 1003: kettleConsents.append(KettleConsent.campaigns)
                case 1004: kettleConsents.append(KettleConsent.ads)
                default: break
                }
            }
            Kettle.shared.revoke(consents: kettleConsents)
            result(true)
        } else {
            result(false)
        }
    }
    
    private func grant(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let args = call.arguments as? [String: Any] else {
            result(FlutterError(code: "KTLERR-2", message: "At least one consent must be provided.", details: nil))
            return
        }
        
        if let consents = args["consents"] as? [Int] {
            if consents.isEmpty {
                result(FlutterError(code: "KTLERR-2", message: "At least one consent must be provided.", details: nil))
                return
            }
            var kettleConsents: [KettleConsent] = []
            for consent in consents {
                switch consent {
                case 1001: kettleConsents.append(KettleConsent.surveys)
                case 1002: kettleConsents.append(KettleConsent.analytics)
                case 1003: kettleConsents.append(KettleConsent.campaigns)
                case 1004: kettleConsents.append(KettleConsent.ads)
                default: break
                }
            }
            Kettle.shared.grant(consents: kettleConsents)
            result(true)
        } else {
            result(false)
        }
    }
    
    private func stop(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let args = call.arguments as? [String: Any] else {
            result(FlutterError(code: "KTLERR-1", message: "At least one module must be provided.", details: nil))
            return
        }
        
        if let modules = args["modules"] as? [Int] {
            if modules.isEmpty {
                result(FlutterError(code: "KTLERR-1", message: "At least one module must be provided.", details: nil))
                return
            }
            
            var kettleModules: [KettleModule] = []
            for module in modules {
                switch module {
                case 1: kettleModules.append(KettleModule.location)
                case 2: kettleModules.append(KettleModule.bluetooth)
                case 3: kettleModules.append(KettleModule.activity)
                default: break
                }
            }
            Kettle.shared.stop(with: kettleModules)
            result(true)
        } else {
            result(false)
        }
    }
    
    private func start(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let args = call.arguments as? [String: Any] else {
            result(FlutterError(code: "KTLERR-1", message: "At least one module must be provided.", details: nil))
            return
        }
        
        if let modules = args["modules"] as? [Int] {
            if modules.isEmpty {
                result(FlutterError(code: "KTLERR-1", message: "At least one module must be provided.", details: nil))
                return
            }
            
            var kettleModules: [KettleModule] = []
            for module in modules {
                switch module {
                case 1: kettleModules.append(KettleModule.location)
                case 2: kettleModules.append(KettleModule.bluetooth)
                case 3: kettleModules.append(KettleModule.activity)
                default: break
                }
            }
            Kettle.shared.start(with: kettleModules)
            result(true)
        } else {
            result(false)
        }
    }
}
